/******************************************************************************
 * Filename: config.c
 *
 * Created: 2012/04/30 20:50
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include "core.h"

int _config_show_bblocks = 0;
int _config_instr_bytes_per_line = 7;
int _config_gidis_arrow_size = 3;
int _config_gidis_show_aliases = 1;
int _config_gidis_view_layouts[][5] = { { 5, 8, 23, 65, 120 }, {5, 8, -1, 30, 120} };

int idis_config_get_layout(const char *name ) {
	PyObject *mod = PyImport_ImportModule("config");
	assert(mod);
	PyObject *val = PyObject_GetAttrString(mod, name);
	if (!PyList_Check(val)) {
		debugf("Reading configuration value %s failed, defaulting.", name);
		return -1;
	}
	Py_ssize_t size = PyList_Size(val);
	if (size != 2) {
		debugf("Configuration value %s is required to be int[2][5].", name);
		return -1;
	}
	for (int i = 0; i < 2; i++) {
		PyObject *list = PyList_GetItem(val, i);
		if (!PyList_Check(val)) {
			debugf("Configuration value %s is required to be int[2][5].", name);
			return -1;
		}
		for (int j = 0; j < 5; j++) {
			PyObject *item = PyList_GetItem(list, j);
			int v = (int) PyInt_AsLong(item);
			_config_gidis_view_layouts[i][j] = v;
			debugf("Setting layout to [%i][%i] := %i", i, j, v);
		}
	}
	return 0;
}

int idis_config_get_int(const char *name, int *dest) {
	PyObject *mod = PyImport_ImportModule("config");
	assert(mod);
	PyObject *val = PyObject_GetAttrString(mod, name);
	long ret = PyInt_AsLong(val);
	if ((ret == -1) && (PyErr_Occurred() != NULL)) {
		debugf("Reading configuration value %s failed, defaulting.", name);
		return -1;
	}
	(*dest) = (int) ret;
	return 0;
}

int idis_config_load() {
	PyRun_SimpleString("import config");
	idis_config_get_int("show_bblocks", &_config_show_bblocks);
	idis_config_get_layout("layout");
	return 0;
}
