/******************************************************************************
 * Filename: constants.c
 * Description:
 *
 * Version: 1.0
 *
 * Author: Ladislav Láska
 * e-mail: ladislav.laska@gmail.com
 *
 ******************************************************************************/

#include <gelf.h>
#include <assert.h>

#include "constants.h"
#include "instr.h"
#include "util.h"


char *strptype(int ptype) {
	char *s;
#define C(V) case PT_##V: s = #V; break
	switch (ptype) {
		C(NULL);
		C(LOAD);
		C(DYNAMIC);
		C(INTERP);
		C(NOTE);
		C(SHLIB);
		C(PHDR);
		C(TLS);
		C(NUM);
		C(LOOS);
		C(GNU_EH_FRAME);
		C(GNU_STACK);
		C(GNU_RELRO);
		C(PAX_FLAGS);
		C(SUNWBSS);
		C(SUNWSTACK);
		C(LOPROC);
		C(HIPROC);
		default:
		s = "unknown";
		break;
	}
	return s;
#undef C
}

char *strtype(int type) {
	char *s;
#define C(V) case ELF_T_##V: s = #V; break
	switch (type) {
		C(BYTE);
		C(ADDR);
		C(DYN);
		C(EHDR);
		C(HALF);
		C(OFF);
		C(PHDR);
		C(RELA);
		C(REL);
		C(SHDR);
		C(SWORD);
		C(SYM);
		C(WORD);
		C(XWORD);
		C(SXWORD);
		C(VDEF);
		C(VDAUX);
		C(VNEED);
		C(VNAUX);
		C(NHDR);
		C(SYMINFO);
		C(MOVE);
		C(LIB);
		C(GNUHASH);
		C(AUXV);
		default:
		s = "unknown";
		break;
	}
	return s;
#undef C
}

char _strprefix_buf[32];

char *strprefix(int pf, int format) {
	char *s;
#define C(V) case PF_##V: s = #V; break;
	switch (pf) {
		C(NULL);
		C(REX);
		C(REX_B);
		C(REX_X);
		C(REX_R);
		C(REX_W);
		C(FS);
		C(ALTER);
		C(GS);
		C(OPSIZE);
		C(ADDRSIZE);
		C(LOCK);
		C(REPZ);
		C(REPNZ);
		C(0F);
	}
	if (format & IFMT_ATT) {
		idis_memlower(_strprefix_buf, s);
		return _strprefix_buf;
	} else
		return s;
	return s;
#undef C
}

char _strreg_buf[32];

char *strreg(int reg, int format) {
	if (reg == R_ZERO) return "";
	char *s;
#define C(NAME) case R_##NAME: s = #NAME; break;
#define CX(NAME) case R_X_##NAME: s = #NAME; break;
	switch (reg) {
			/* 32 bit registers in 64bit mode */
	C(EAX)  
	C(ECX)  
	C(EDX)  
	C(EBX)
	C(ESP)  
	C(EBP)  
	C(ESI)  
	C(EDI)
	C(R8D)  
	C(R9D)  
	C(R10D) 
	C(R11D)
	C(R12D) 
	C(R13D) 
	C(R14D) 
	C(R15D)

	/* 64bit registers */
	C(RAX) 
	C(RCX) 
	C(RDX) 
	C(RBX)
	C(RSP) 
	C(RBP) 
	C(RSI) 
	C(RDI)
	C(R8)  
	C(R9)  
	C(R10) 
	C(R11)
	C(R12) 
	C(R13) 
	C(R14) 
	C(R15)

	/* 16bit registers */
	C(AX)   
	C(CX)   
	C(DX)   
	C(BX)
	C(SP)   
	C(BP)   
	C(SI)   
	C(DI)
	C(8D)   
	C(9D)   
	C(R10W) 
	C(R11W)
	C(R12W) 
	C(R13W) 
	C(R14W) 
	C(R15W)

	/* 8bit registers */
	C(AL)   
	C(CL)   
	C(DL)   
	C(BL)
	C(AH)   
	C(CH)   
	C(DH)   
	C(BH)
	C(R8B)  
	C(R9B)  
	C(R10B) 
	C(R11B)
	C(R12B) 
	C(R13B)
	C(R14B) 
	C(R15B)

	/* Special registers accessible with REX prefix */
	CX(AL)   
	CX(CL)   
	CX(DL)   
	CX(BL)
	CX(SPL)  
	CX(BPL)  
	CX(SIL)  
	CX(DIL)
	CX(R8B)  
	CX(R9B)  
	CX(R10B) 
	CX(R11B)
	CX(R12B) 
	CX(R13B) 
	CX(R14B) 
	CX(R15B)

	/* Special registers */
	C(RIP)


		default: s = "_R_INVD_RANGE";
	}
#undef C
#undef CX

	if (format & IFMT_ATT) {
		idis_memlower(_strreg_buf+1, s);
		_strreg_buf[0] = '%';
		return _strreg_buf;
	} else
		return s;
}
