/******************************************************************************
 * Filename: expr.h
 *
 * Created: 2012/02/13 00:28
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#ifndef __EXPR_H__
#define __EXPR_H__

#include <stdlib.h>
#include "instr.h"
#include "program.h"

#define __EX_BINARY EX_PLUS
#define __EX_UNARY EX_REG
#define __EX_OPERATOR EX_DEREF

enum EX_OP {
	EX_UNKNOWN,
	EX_REG,
	EX_CONST,
	EX_DEREF,
	EX_PLUS,
	EX_MINUS,
	EX_MUL,
	EX_DIV,
	EX_ASSIGN,
	__EX_LAST
};

struct expr {
	unsigned short int type;
	unsigned short int opn;
#ifndef SWIG
	union {
		struct expr **ops;
		value_t val;
	};
#else
	/* Swig can't handle anonymous union, will handle it manually in the wrapper. */
	struct expr **ops;
#endif
};

struct expr *idis_operand_enumerate_partial( struct op *op );
struct expr *idis_expr_new(int opn);
struct expr *idis_expr_new_value( value_t val );
struct expr *idis_expr_new_reg( value_t reg );

struct expr *idis_expr_new_unary( unsigned short int type, struct expr *op );
struct expr *idis_expr_new_binary( unsigned short int type, 
	struct expr *op1, struct expr *op2 );

char *idis_strexpr( struct expr *ex );
void idis_expr_free( struct expr *ex );

unsigned int idis_expr_hash( struct expr *ex );
int idis_expr_compare( struct expr *ex1, struct expr *ex2 );
int idis_expr_equal( struct expr *ex1, struct expr *ex2 );

int idis_expr_enumerate_error();
value_t idis_expr_enumerate( struct expr *expr, struct program *prg, struct instr *instr );
struct expr *idis_expr_simplify( struct expr *expr, struct program *prg, struct instr *instr );

#endif
