/******************************************************************************
 * Filename: instr.h
 *
 * Created: 2011/10/16 21:55
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#ifndef __INSTR_H__
#define __INSTR_H__

#include "instr_data.h"

extern struct instr_def **_opcds[];
extern struct instr_def **_opcds0f[];
extern int _prefix_transl[][2];
extern struct instr_def _invd;

typedef unsigned long long int vmaddr_t;
typedef unsigned long long int value_t;

enum OP_TYPE {
	OT_UNKNOWN,
	OT_REG,
	OT_IMMEDIATE,
	OT_DISPLACEMENT,
	OT_INDIRECT,
	OT_INDIRECT_DISP,
	OT_JUMP_REL,
	OT_STACK,
	OT_SIB,
	__OT_LAST
};

#define _REG32 R_EAX
#define _REG16 R_AX
#define _REG64 R_RAX
#define _REG8 R_AL
#define _REG8_REX R_X_AL
#define _REG_SPECIAL R_RIP

enum REGISTER {
	/* 8bit registers */
	R_AL,   R_CL,   R_DL,   R_BL,
	R_AH,   R_CH,   R_DH,   R_BH,
	R_R8B,  R_R9B,  R_R10B, R_R11B,
	R_R12B, R_R13B, R_R14B, R_R15B,

	/* Special registers accessible with REX prefix */
	R_X_AL,   R_X_CL,   R_X_DL,   R_X_BL,
	R_X_SPL,  R_X_BPL,  R_X_SIL,  R_X_DIL,
	R_X_R8B,  R_X_R9B,  R_X_R10B, R_X_R11B,
	R_X_R12B, R_X_R13B, R_X_R14B, R_X_R15B,

	/* 16bit registers */
	R_AX,   R_CX,   R_DX,   R_BX,
	R_SP,   R_BP,   R_SI,   R_DI,
	R_8D,   R_9D,   R_R10W, R_R11W,
	R_R12W, R_R13W, R_R14W, R_R15W,

	/* 32 bit registers in 64bit mode */
	R_EAX,  R_ECX,  R_EDX,  R_EBX,
	R_ESP,  R_EBP,  R_ESI,  R_EDI,
	R_R8D,  R_R9D,  R_R10D, R_R11D,
	R_R12D, R_R13D, R_R14D, R_R15D,

	/* 64bit registers */
	R_RAX, R_RCX, R_RDX, R_RBX,
	R_RSP, R_RBP, R_RSI, R_RDI,
	R_R8,  R_R9,  R_R10, R_R11,
	R_R12, R_R13, R_R14, R_R15,

	/* Special registers */
	R_RIP,

	/* Zero register. This is used in SIB, denoting that no register is used. */
	R_ZERO,

	/* Invalid register */
	R_INVD,
	__R_LAST
};

enum {
	PF_NULL 	= 0,
	PF_REX 		= 1 << 0,
	PF_REX_B 	= 1 << 1,
	PF_REX_X 	= 1 << 2,
	PF_REX_R 	= 1 << 3,
	PF_REX_W 	= 1 << 4,
	PF_FS 		= 1 << 5,
	PF_ALTER 	= 1 << 6,
	PF_GS 		= 1 << 7,
	PF_OPSIZE 	= 1 << 8,
	PF_ADDRSIZE 	= 1 << 9,
	PF_LOCK 	= 1 << 10,
	PF_REPZ 	= 1 << 11,
	PF_REPNZ 	= 1 << 12,
	PF_0F 		= 1 << 13,
	__PF_LAST 	= 1 << 14
};

enum IFLAGS {
	IF_MODRM 	= 1 << 0,
	IF_SIB 		= 1 << 1,
	__IF_LAST	= 1 << 2,
};

enum INSTR_FIELDS {
	FI_MOD,
	FI_REG,
	FI_RM,
	FI_SCALE,
	FI_INDEX,
	FI_BASE,
	__FI_LAST
};

#define IFMT_DEFAULT (IFMT_ATT | IFMT_IMM_SIGNED)

enum {
	IFMT_ATT   = 1 << 0,
	IFMT_INTEL = 1 << 1,
	IFMT_IMM_SIGNED = 1 << 2
};

enum {
	BITS8 = 1,
	BITS16 = 2,
	BITS32 = 4,
	BITS64 = 8
};

struct instr_llist {
	struct instr *instr;
	struct instr_llist *next;
};

/* A structure for decoded operand, with all details */
struct op {
	int type;	/* Operand type, as defined by OP_TYPE enum. */
	long long int val;	/* Operand value */
	unsigned char scale;
	unsigned char index;
	unsigned char base;
	long long int disp;
	/* Some kind of expression needed here, RPN perhaps? */
};

enum USERDATA_TYPE {
	UDT_UNKNOWN,
	UDT_COMMENT,
	UDT_ANNOTATE_HOOK,
	UDT_CUSTOM,
	__UDT_LAST
};

struct userdata {
	unsigned int type;
	unsigned int owner;
	unsigned char serialize;
	void *callback;
	void *data;
};

/* A struct for decoded instruction */
struct instr {
	_instr_def_t *def; 	/* Pointer to global definition table */
	unsigned char *raw;	/* Raw instruction data */
	unsigned char raw_length; /* Length of raw instruction in bytes */
	unsigned char modrm;
	unsigned char sib;
	unsigned int flags;
	unsigned int prefixes;
	vmaddr_t vmaddr;
	struct op **ops;		/* NULL-terminated list of operands */
	int opn;
	struct instr_llist *jumpsto;
	struct instr_llist *jumpedfrom;

	struct userdata **userdata;
	unsigned int userdatan;

	unsigned int style;
};

struct instr *alloc_instr();
void free_instr( struct instr *instr );
unsigned char idis_parse_modrm( struct instr *instr, int field );
unsigned char idis_parse_sib( struct instr *instr, int field );
struct op *idis_push_operand2( struct instr *instr, int type, long long int val, long long int disp );
struct op *idis_push_operand( struct instr *instr, int type, long long int val );
int idis_get_regoff(struct instr *instr, struct op_def *op);
int idis_get_addroff(struct instr *instr);
int idis_get_opsize(struct instr *instr, struct op_def *op);
int idis_get_addressing(struct instr *instr);
int idis_get_opextend(struct instr *instr, struct op_def *op);
int idis_instr_basebits(struct instr *instr);

char *strinstr(struct instr *instr);
char *strinstrf(struct instr *instr, int format);
char *idis_instr_tostr_arg();
char *idis_instr_tostr(struct instr *instr, int format);
struct expr* idis_instr_tostr_arg_expr();
int idis_instr_tostr_arg_index();

char *idis_stropf(struct instr *instr, struct op *o, int format);
char *idis_strop(struct instr *instr, struct op *op);
struct op *idis_instr_src( struct instr *instr );
struct op *idis_instr_dst( struct instr *instr );

#endif
