/******************************************************************************
 * Filename: mem.c
 *
 * Created: 2012/05/21 19:57
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include <ctype.h>
#include "mem.h"
#include "debug.h"

unsigned char idis_mem_read_byte( struct program *prg, vmaddr_t vmaddr ) {
	struct section *sec = idis_lookup_section_by_addr( prg, vmaddr );
	if (sec == NULL) return 0;

	vmaddr_t offset = vmaddr - sec->addr;
	return (*(sec->data+offset)) & 0xff;
}

char _idis_mem_strbuf[128];

char *idis_mem_read_string( struct program *prg, vmaddr_t vmaddr ) {
	struct section *sec = idis_lookup_section_by_addr( prg, vmaddr );
	if (sec == NULL) return 0;

	vmaddr_t offset = vmaddr - sec->addr;
	for (int i = 0; i < 127; i++) {
		char c = (*(sec->data+offset+i)) & 0xff;
		_idis_mem_strbuf[i] = c;
		if (c == 0) {
			/* Check if it's string */
			int printable = 0;
			for (int j = 0; j < i; j++) {
				if (isprint(_idis_mem_strbuf[j]) || isspace(_idis_mem_strbuf[j])) printable++;
			}
			if (i-printable > 0) return NULL;
			return _idis_mem_strbuf;
		}
	}
	return NULL;
}

long long int idis_mem_read_long( struct program *prg, vmaddr_t vmaddr ) {
	struct section *sec = idis_lookup_section_by_addr( prg, vmaddr );
	if (sec == NULL) return 0;

	vmaddr_t offset = vmaddr - sec->addr;
	unsigned long long int val = 0;
	for (int i = 0; i < 8; i++) {
		val |= ((*(sec->data+offset+i)) & 0xff) << (i*8);
	}

	return val;
}
