/******************************************************************************
 * Filename: program.c
 *
 * Created: 2011/11/27 23:45
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include <stdlib.h>

#include "program.h"
#include "instr.h"
#include "debug.h"
#include "assumption.h"
#include "symbol.h"
	
struct program *idis_alloc_program() {
	struct program *prg = malloc(sizeof(struct program));
	memset(prg, 0, sizeof(struct program));
	return prg;
}

void idis_free_program(struct program *prg) {
	free(prg);
}

struct instr *idis_lookup_instr_by_addr( struct program *prg, vmaddr_t vmaddr ) {
	/* First, find correct section */
	for (int i = 0; prg->sections[i] != NULL; i++) {
		vmaddr_t sbegin = prg->sections[i]->addr;
		vmaddr_t send = sbegin + prg->sections[i]->data_size; 
		if ((vmaddr >= sbegin) && (vmaddr < send)) {
			for (int j = 0; j < prg->sections[i]->instrn; j++) {
				if (prg->sections[i]->instrs[j]->vmaddr == vmaddr)
					return prg->sections[i]->instrs[j];
				/*if (prg->sections[i]->instrs[j]->vmaddr > vmaddr) {
					warn("Jump inside instruction detected. Maybe disassembling incorrect or obfuscated code?");
					return NULL;
				}*/
			}
		}
	}
	warn("Jump destination not found, probably jump to dynamic lib.");
	return NULL;
}

struct section *idis_lookup_section_by_name( struct program *prg, const char *name ) {
	for (int i = 0; prg->sections[i] != NULL; i++) {
		if (strcmp(name, prg->sections[i]->name) == 0)
			return prg->sections[i];
	}
	return NULL;
}

struct section *idis_lookup_section_by_addr( struct program *prg, vmaddr_t vmaddr ) {
	for (int i = 0; prg->sections[i] != NULL; i++) {
		struct section *s = prg->sections[i];
		if ((vmaddr >= s->addr) && (vmaddr < (s->addr + s->data_size)))
			return s;
	}
	return NULL;
}

struct symbol *idis_instr_branch_symbol( struct program *prg, struct instr *instr ) {
	struct op *op;
	for (int j = 0; (op = instr->ops[j]) != NULL; j++) {
		if (op->type == OT_JUMP_REL) {
			struct assumption *as = idis_assumption_getfor( prg, instr, AK_REG, R_RIP);
			vmaddr_t jaddr = as->value.imm + op->val;
			struct symbol *symb = idis_lookup_symbol_by_addr( prg, jaddr );
			return symb;
		}
	}
	return NULL;
}
