/******************************************************************************
 * Filename: program.h
 * Description:
 *
 * Version: 1.0
 * Created: Sep 11 2011 20:27:45
 * Last modified: Sep 11 2011 20:27:45
 *
 * Author: Ladislav Láska
 * e-mail: ladislav.laska@gmail.com
 *
 ******************************************************************************/
#ifndef _CORE_PROGRAM_H_
#define _CORE_PROGRAM_H_

#include <gelf.h>

#include "instr.h"

enum {
	SF_EXE = 	1 << 0
};

struct section {
	char *name;
	vmaddr_t addr;
	unsigned char *data;
	size_t data_size;
	struct instr **instrs;
	int instrn;
	int type;
	unsigned int flags;
	struct program *prg;
};

struct symbol {
	char *name;
	size_t addr;
};

struct bblock_llist {
	struct bblock *bblock;
	struct bblock_llist *next;
};

struct bblock {
	struct instr *firsti;
	struct instr *lasti;
	struct instr **instrs;
	int instrn;
	vmaddr_t vmaddr;
	struct bblock_llist *jumpsto;
	struct bblock_llist *jumpedfrom;
};

struct program {
	char *file;
	Elf *elf;
	int elfclass;
	char *abi;
	struct section **sections;
	int secn;
	struct symbol **symbs;
	int symbn;

	struct bblock **bblocks;
	int bblockn;
	struct seri_data *seri_data;
};

/* Serialization data. */
enum {
	Y_NULL = 0,
	Y_SEQ = 1,
	Y_MAP = 2,
	Y_SCALAR = 3
};

struct seri_data {
	int type;
	union {
		char *val;
		struct seri_data *obj;
	} data;
	struct seri_data *next;
};

struct program *idis_alloc_program();
void idis_free_program(struct program *prg);

struct instr *idis_lookup_instr_by_addr( struct program *prg, vmaddr_t vmaddr );
struct section *idis_lookup_section_by_name( struct program *prg, const char *name );
struct section *idis_lookup_section_by_addr( struct program *prg, vmaddr_t vmaddr );

/* Returns symbol where instruction branches, or NULL if none exists. */
struct symbol *idis_instr_branch_symbol( struct program *prg, struct instr *instr );

#endif
