/******************************************************************************
 * Filename: python.h
 *
 * Created: 2012/01/12 20:54
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#ifndef __CORE_PYTHON_H__
#define __CORE_PYTHON_H__

#include <Python.h>

#include "program.h"

enum {
	PHT_INIT,
	PHT_RUN,
	PHT_FOR_INSTR,
	PHT_FOR_BBLOCK,
	PHT_DISASSEMBLE,
	PHT_FINALIZE,
	PHT_SERIALIZE,
	PHT_DESERIALIZE
};

enum {
	PHS_STD = 0,
	PHS_SERIALIZE,
	PHS_DESERIALIZE
};

struct python_module_llist {
	char *name;
	PyObject *obj;
	struct python_module_llist *next;
};

#define idis_python_call(mod, fn, ...) _idis_python_call(mod, fn, NARG(__VA_ARGS__), ##__VA_ARGS__)

int _idis_python_call(const char *module_name, const char *fn_name, int argn, ...);
int idis_python_call_hook(struct program *prg, const char *module_name, int hook_type);
void idis_python_call_hooks( struct program *prg, int set );
int idis_python_serialize( struct program *prg );
int idis_python_load_module( char *module_name );

void idis_python_init();
void idis_python_fini();

/* 
 * These are helper function to export various structs to python.
 *
 * The code is in python.i
 */
PyObject *idis_python_export_program( struct program *prg );
PyObject *idis_python_export_section( struct section *sec );
PyObject *idis_python_export_symbol( struct symbol *sym );
PyObject *idis_python_export_bblock( struct bblock *bblock );
PyObject *idis_python_export_instr( struct instr *instr );
PyObject *idis_python_export_op( struct op *op );
PyObject *idis_python_export_str( char *str );
PyObject *idis_python_export_userdata( struct userdata *ud );

void idis_python_append_path(char *path, int autload);

#endif
