%module idis

%rename("%(regex:/idis_(.*)/\\1/)s") "";

%include "wrap_array.i"

/* wrap_array(parent_struct,array_type,array_data,array_length) */
%wrap_array(program,symbol,symbs,symbn);
%wrap_array(program,bblock,bblocks,bblockn);
%wrap_array(program,section,sections,secn);
%wrap_array(section,instr,instrs,instrn);
%wrap_array(bblock,instr,instrs,instrn);
%wrap_array(instr,op,ops,opn);
%wrap_array(expr,expr,ops,opn);

%{
#define SWIG_FILE_WITH_INIT
#include "../core.h"

PyObject *idis_python_export_program( struct program *prg ) {
    return SWIG_NewPointerObj( prg, SWIGTYPE_p_program, 0 );
}

PyObject *idis_python_export_section( struct section *sec) {
    return SWIG_NewPointerObj( sec, SWIGTYPE_p_section, 0 );
}

PyObject *idis_python_export_symbol( struct symbol *sym) {
    return SWIG_NewPointerObj( sym, SWIGTYPE_p_symbol, 0 );
}

PyObject *idis_python_export_bblock( struct bblock *bblock) {
    return SWIG_NewPointerObj( bblock, SWIGTYPE_p_bblock, 0 );
}

PyObject *idis_python_export_instr( struct instr *instr) {
    return SWIG_NewPointerObj( instr, SWIGTYPE_p_instr, 0 );
}

PyObject *idis_python_export_op( struct op *op ) {
    return SWIG_NewPointerObj( op, SWIGTYPE_p_op, 0 );
}

PyObject *idis_python_export_userdata( struct userdata *ud ) {
    return SWIG_NewPointerObj( ud, SWIGTYPE_p_userdata, 0 );
}

PyObject *idis_python_export_str( char *str ) {
    return PyString_FromString(str);
}

signed long long idis_signed( unsigned long long val ) {
    return (long long) val;
}
%}

signed long long idis_signed( unsigned long long val );

%include "../core/core.h"
%include "../core/error.h"
%include "../core/disasm.h"
%include "../core/symbol.h"
%include "../core/program.h"
%include "../core/debug.h"
%include "../core/messages.h"
%include "../core/constants.h"
%include "../core/instr.h"
%include "../core/util.h"
%include "../core/instr_data.h"
%include "../core/assumption.h"
%include "../core/expr.h"
%include "../core/serializer.h"
%include "../core/mem.h"
%include "../core/reg.h"
%include "../core/userdata.h"
%include "../core/bblock.h"


%inline %{
    extern char *idis_new_str(char *str);
    char *idis_new_str( char *str ) {
        return strdup(str);
    }
%}

%define %memberof_r(CLASS,FUNCTION,RENAME)
%extend CLASS { 
%pythoncode %{
def RENAME ( self, *args ):
    return FUNCTION (self, *args)
%}
}
%enddef

%define %memberof(CLASS,FUNCTION)
%memberof_r(CLASS,FUNCTION,FUNCTION)
%enddef

%define %memberof_v(CLASS,ORIG,REN)
%extend CLASS {
%pythoncode %{
    __swig_getmethods__["REN"] = lambda self: ORIG
%}
}
%enddef

%memberof(program, add_symbol);
%memberof(program, lookup_symbol);
%memberof(program, lookup_symbol_by_addr);
%memberof(program, lookup_symbol_nearest);
%memberof(program, lookup_symbol_by_name);
%memberof(program, finalize_symbols);
%memberof(program, assumption_getfor);
%memberof(program, lookup_section_by_name);
%memberof(program, lookup_section_by_addr);
%memberof(program, lookup_instr_by_addr);
%memberof(program, lookup_bblock_by_instr);

%memberof_r(instr, instr_add_annotate_hook, add_annotate_hook);
%memberof_r(instr, instr_add_userdata, add_userdata);
%memberof_r(instr, instr_add_comment, add_comment);
%memberof_r(instr, strinstr, __str__);
%memberof_v(instr, self._def.mnem.lower(), mnem);
%memberof_v(instr, self._def.desc, desc);
%memberof_v(instr, instr_src(self), src);
%memberof_v(instr, instr_dst(self), dst);

%memberof_r(expr, strexpr, __str__);
%memberof_r(expr, 0 == expr_compare, __eq__);
%memberof_r(expr, expr_free, free);

%memberof_v(op, operand_enumerate_partial(self), expr);

/* Wrap anonymous union in expr_t */
%extend expr {
%pythoncode %{
  __swig_getmethods__["val"] = lambda obj: _idis.expr_value(obj)
%}
}

%inline %{
    value_t _idis_expr_value( struct expr *ex ) { return ex->val; }
%}

/* Wrap .expr member of struct op */
%extend op {
%pythoncode %{
    __swig_getmethods__["expr"] = lambda obj: _idis.operand_enumerate_partial( obj )
%}
}

%pythoncode %{

import sys, os, pkgutil

def new_handle():
    return python_new_handle();
def serialize(tbl):
    seri_mapping_start();
    for k in tbl.keys():
       if isinstance( tbl[k], dict ):
           seri_scalar(str(k));
           serialize(tbl[k]);
       else:
           seri_mapping_item(str(k), str(tbl[k]));
    seri_mapping_end();

def list_modules(path):
    sys.path.insert(0, os.path.expanduser(path));
    return [name for _, name, _ in pkgutil.walk_packages([path])];
%}
