/******************************************************************************
 * Filename: symbol.c
 *
 * Created: 2011/11/26 13:20
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include <assert.h>
#include <stdlib.h>

#include "symbol.h"
#include "debug.h"

int _idis_symbol_addrcomp_fn( const struct symbol **a, const struct symbol **b ) {
	return ((*a)->addr - (*b)->addr);
}

struct symbol *idis_lookup_symbol( struct program *prg, size_t addr ) {
	return idis_lookup_symbol_by_addr(prg, addr);
}

struct symbol *idis_lookup_symbol_nearest( struct program *prg, size_t addr ) {
	/* Binary search in sorted array */
	int l = 0,r = prg->symbn-1;
	while (l < r) {
		int i = (r - l)/2 + (r - l)%2;
		if (prg->symbs[l+i]->addr <= addr) l += i;
		else r -= i;
	}
	if (prg->symbs[l]->addr <= addr) return prg->symbs[l];
	else return NULL;
}

struct symbol *idis_lookup_symbol_by_addr( struct program *prg, size_t addr ) {
	/* Binary search in sorted array */
	int l = 0,r = prg->symbn-1;
	while (l < r) {
		int i = (r - l)/2 + (r - l)%2;
		if (prg->symbs[l+i]->addr <= addr) l += i;
		else r -= i;
	}
	if (prg->symbs[l]->addr == addr) return prg->symbs[l];
	else return NULL;
}

struct symbol *idis_lookup_symbol_by_name( struct program *prg, char *name ) {
	assert(0); /* Not supported yet */
}

void idis_finalize_symbols( struct program *prg ) {
	qsort(prg->symbs, prg->symbn, sizeof(struct symbol*), (__compar_fn_t) &_idis_symbol_addrcomp_fn);
}

struct symbol *idis_add_symbol( struct program *prg ) {
	struct symbol *s = malloc(sizeof(struct symbol));
	memset(s, 0, sizeof(struct symbol));
	struct symbol **symbs = realloc(prg->symbs, (1+prg->symbn)*sizeof(struct symbol*));
	if (s == NULL) {
		warn("Could not malloc() space for new symbol.");
		return NULL;
	}

	prg->symbs = symbs;
	symbs[prg->symbn++] = s;
	return s;
}

char _idis_strsym_buf[128];

char *idis_strsym( struct symbol *sym ) {
	snprintf(_idis_strsym_buf, 128, "<%s>", sym->name);
	return _idis_strsym_buf;
}

char *idis_strsym_rel( struct symbol *sym, vmaddr_t relto ) {
	if (relto == sym->addr) return idis_strsym( sym );

	vmaddr_t diff;
	if (sym->addr < relto) diff = relto-sym->addr;
	else diff = sym->addr-relto;
	snprintf(_idis_strsym_buf, 128, "<%s%c0x%llx>", sym->name, (sym->addr < relto) ? '+' : '-', diff);
	return _idis_strsym_buf;
}
