/******************************************************************************
 * Filename: userdata.c
 *
 * Created: 2012/05/23 23:13
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include "userdata.h"

struct userdata *idis_instr_add_userdata( struct instr *instr, unsigned int type ) {
	if (instr->userdatan == 0) {
		instr->userdata = malloc(sizeof(struct userdata*));
	} else {
		struct userdata **newptr = realloc(instr->userdata, 
			sizeof(struct userdata*)*(instr->userdatan+1));
		assert(newptr);
		instr->userdata = newptr;
	}

	struct userdata *d = malloc(sizeof(struct userdata));
	memset(d, 0, sizeof(struct userdata));
	d->type = type;
	instr->userdata[instr->userdatan++] = d;

	return d;
}

void idis_instr_add_annotate_hook( struct instr *instr, PyObject *callback, PyObject *data ) {
	Py_INCREF(data);
	struct userdata *ud = idis_instr_add_userdata(instr, UDT_ANNOTATE_HOOK);
	ud->callback = (void *) callback;
	ud->data = (void *) data;
}

void idis_instr_add_comment( struct instr *instr, char *ptr ) {
	struct userdata *ud = idis_instr_add_userdata( instr, UDT_COMMENT );
	ud->data = (void *) strdup(ptr);
}

void idis_instr_free_userdata( struct userdata *ud ) {
	switch (ud->type) {
		case UDT_ANNOTATE_HOOK:
			Py_DECREF((PyObject *) ud->data);
			break;
		case UDT_COMMENT:
			free(ud->data);
			break;
	}
	free(ud);
}
