/******************************************************************************
 * Filename: util.c
 *
 * Created: 2011/11/17 13:01
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include <stddef.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

#include "util.h"
#include "instr.h"

void idis_memset32(unsigned char *dst, unsigned int patt, size_t size) {
	assert(size % 4 == 0);
	unsigned int *ptr = (unsigned int *) dst;
	for (int i = 0; i < (size%4); i++)
		ptr[i] = patt;
}

int idis_memlower(char *dst, char *src) {
		int i;
		for (i = 0; src[i] != 0; i++) {
			dst[i] = src[i];
			if (src[i] >= 'A' && src[i] <= 'Z')
				dst[i] += 'a' - 'A';
		}
		dst[i] = 0;
		return i;
}

#define STRADDRBUF_SIZE 128
char _straddrbuf[STRADDRBUF_SIZE];

char *straddrf(vmaddr_t addr, int format) {
	char *sign;
	if (addr < 0) {
		addr *= -1;
		sign = "-";
	} else sign = "";

	int ret = snprintf(_straddrbuf, STRADDRBUF_SIZE, "%s0x%llx", sign, addr);
	assert(ret > 0);
	return _straddrbuf;
}

#define STRRELBUF_SIZE 128
char _strrelbuf[STRRELBUF_SIZE];

char *strrelf(long long int rel, int format) {
	char *sign;
	if (rel < 0) {
		rel *= -1;
		sign = "-";
	} else sign = "+";

	int ret = snprintf(_strrelbuf, STRRELBUF_SIZE, "%s0x%llx", sign, rel);
	assert(ret > 0);
	return _strrelbuf;
}

#define STRDISPBUF_SIZE 128
char _strdispbuf[STRDISPBUF_SIZE];

char *strdispf(long long int disp, int format) {
	char *sign;
	if (disp < 0) {
		disp *= -1;
		sign = "-";
	} else sign = "";

	int ret = snprintf(_strdispbuf, STRDISPBUF_SIZE, "%s0x%llx", sign, disp);
	assert(ret > 0);
	return _strdispbuf;
}

#define STRIMMBUF_SIZE 128
char _strimmbuf[STRIMMBUF_SIZE];

char *strimmf(long long int imm, int format) {
	char *sign;
	char *dollar;
	if ((format & IFMT_IMM_SIGNED) && (imm < 0)) {
		imm *= -1;
		sign = "-";
	} else sign = "";

	if (format & IFMT_ATT) 
		dollar = "$";
	else dollar = "";

	int ret = snprintf(_strimmbuf, STRIMMBUF_SIZE, "%s%s0x%llx", sign, dollar, imm);
	assert(ret > 0);
	return _strimmbuf;
}

char *strimm( long long int imm ) {
	return strimmf(imm, IFMT_DEFAULT);
}

