/******************************************************************************
 * Filename: alias.c
 *
 * Created: 2012/04/14 21:47
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include "view.h"

#include "alias.h"
#include "../config.h"

GHashTable *_gidis_alias_htable = NULL;

void gidis_alias_init() {
	if (_gidis_alias_htable != NULL) g_hash_table_destroy(_gidis_alias_htable);
	_gidis_alias_htable = g_hash_table_new_full( 
		(GHashFunc) idis_expr_hash, 
		(GEqualFunc) idis_expr_equal, 
		(GDestroyNotify) idis_expr_free, 
		(GDestroyNotify) free
	);

}

void gidis_alias_free() {
	if (_gidis_alias_htable != NULL) g_hash_table_destroy(_gidis_alias_htable);
	_gidis_alias_htable = NULL;
}

void gidis_alias_expr(char *name, struct expr *expr) {
	g_hash_table_insert(_gidis_alias_htable, expr, strdup(name));
	gidis_view_rebuild_linebuf();
}

int gidis_alias_new(char *name, char *expr) {
	if (expr[0] == '$') {
		/* Expression is value */
		char *end;
		value_t val = strtoll(expr+1, &end, 0);
		if (end[0] != 0) {
			debugf("Error parsing alias (invalid constant): %s -> %s", expr, name);
			return ERR_GA_INVALID_CONSTANT;
		} else {
			struct expr *ex = idis_expr_new_value(val);
			gidis_alias_expr(name, ex);
			return 0;
		}
	} else return ERR_GA_EXPR_NOT_SUPPORTED;
}

char *gidis_alias_lookup(struct expr *ex) {
	if (_gidis_alias_htable == NULL) return NULL;
	char *res = g_hash_table_lookup(_gidis_alias_htable, ex);
	return res;
}

int gidis_alias_enabled() {
	return _config_gidis_show_aliases;
}
