/******************************************************************************
 * Filename: context_menu.c
 *
 * Created: 2012/05/03 13:10
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include "view.h"
#include <gtk/gtk.h>
#include "context_menu.h"

GtkWidget *menu = NULL;
GtkWidget *item = NULL;


struct cm_item *_gidis_cm_handlers;
struct region *_gidis_cm_region;
struct instr_info *_gidis_cm_instr_info;


void _gidis_context_menu_activated(GtkWidget *menuitem, gpointer userdata) {
	struct cm_item *cmi = userdata;
	PyObject *instr = idis_python_export_instr(_gidis_cm_instr_info->instr);
	PyObject *prg = idis_python_export_program(_gidis_prg);
	PyObject *args;

	switch (cmi->type) {
		case CMT_OPERAND:;
			/* Hook format: instr, operand # */
			int opid = _gidis_cm_region->id - 10;
			assert(opid >= 0);
			PyObject *op = idis_python_export_op(_gidis_cm_instr_info->instr->ops[opid]);
			args = PyTuple_New(3);
			PyTuple_SetItem(args, 0, prg);
			PyTuple_SetItem(args, 1, instr);
			PyTuple_SetItem(args, 2, op);
			PyObject_CallObject( cmi->callback.python, args );
			PyErr_Print();
			break;
		case CMT_MNEMONIC:
			/* assert(_gidis_cm_region->id == GRT_MNEMONIC); */
			args = PyTuple_New(2);
			PyTuple_SetItem(args, 0, prg);
			PyTuple_SetItem(args, 1, instr);
			PyObject_CallObject( cmi->callback.python, args );
			PyErr_Print();
			break;
	}
}

int gidis_context_menu_build( int typemask ) {
	/* The menu will destory itself after it vanishes. */
	menu = gtk_menu_new();
	struct cm_item *cmi = _gidis_cm_handlers;
	int items = 0;
	while (cmi != NULL) {
		if (cmi->type & typemask) {
			item = gtk_menu_item_new_with_label(cmi->name);
			g_signal_connect(item, "activate", (GCallback) _gidis_context_menu_activated, cmi);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
			items++;
		}

		cmi = cmi->next;
	}
	if (items > 0) {
		gtk_widget_show_all(menu);
	}
	/* TODO: More cleanup? Manual says "never call g_boject_unref() unless you 
	 * called g_object_ref(), even if you just created the object". Also, 
	 * gtk_menu does not have destory function. */
	
	return items;
}

void gidis_context_menu_popup(struct instr_info *info, struct region *r) {
	if (gidis_context_menu_build( (r->id >= 10) ? CMT_OPERAND : CMT_MNEMONIC ) == 0) return;
	_gidis_cm_region = r;
	_gidis_cm_instr_info = info;
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
}

void gidis_add_context_hook( char *name, int type, PyObject *obj ) {
	assert(PyCallable_Check(obj));
	struct cm_item *i = malloc(sizeof(struct cm_item));
	memset(i, 0, sizeof(struct cm_item));
	i->name = strdup(name);
	i->type = type;
	i->callback.python = obj;
	if (_gidis_cm_handlers == NULL)
		_gidis_cm_handlers = i;
	else {
		struct cm_item *next = _gidis_cm_handlers;
		while (next->next != NULL) next = next->next;
		next->next = i;
	}
}
