/******************************************************************************
 * Filename: dumper.c
 * Description: Dumps contents of current gui view into a file, as is, padded 
 *   by spaces.
 *
 * Created: 2012/03/29 20:51
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include "view.h"

#include <assert.h>
#include <stdio.h>
#include <string.h>

#define BUFFER_LINES 5
#define BUFFER_COLUMNS 256

void gidis_chomp(char *str) {
	char last = -1;
	for (int i = 0; str[i] != 0; i++) {
		switch (str[i]) {
			case ' ':
			case '\t':
			case '\n':
			break;
			default:
				last = i;
		}
	}
	str[last+1] = 0;
}

/* Generic function to dump data into text. If f is NULL, it only does
 * first instruction and leaves it in the buffer. */
char **buffer = NULL;
int _gidis_dump(FILE *f, int first, int last) {
	if (buffer == NULL) {
		buffer = malloc(sizeof(char *)*(BUFFER_LINES+1));
		for (int i = 0; i < BUFFER_LINES; i++) 
			buffer[i] = malloc(sizeof(char)*BUFFER_COLUMNS);
		buffer[BUFFER_LINES] = NULL;
	}

	int wlines;
	if (last < first) last = _gidis_view_instr_info_len-1;

	for (int index = first; index <= last; index++) {
		/* Prepare linebuffer */
		for (int l = 0; l < BUFFER_LINES; l++) {
			memset(buffer[l],' ',BUFFER_COLUMNS);
			buffer[l][BUFFER_COLUMNS-1] = 0;
		}

		struct instr_info *info = &_gidis_view_instr_info[index];
		char *linebuf = info->linebuf;
		/* Parse linebuf */
		int l = 0;
		int c = 0;
		int tab = 0;
		int baseline = 0;
		for (int i = 0; linebuf[i] != 0; i++) {
			switch (linebuf[i]) {
				case '^':
					if (linebuf[i+1] == '/') { i++; continue; }
					else if (strncmp(linebuf+i, "^[baseline]", 11) == 0) {
						baseline = l;
						i += 10;
					} else if (strncmp(linebuf+i, "^[t=", 4) == 0) {
						size_t arglen = strcspn(linebuf+i+4,"]");
						for (int nc = 0; _gidis_view_layout_names[nc] != NULL; nc++) {
							if (strncmp(_gidis_view_layout_names[nc], linebuf+i+4, arglen) == 0) {
								c = _gidis_view_layout[tab = nc];
								l = baseline;
							}
						}
						/* Ignore if not found... */
						i += 4 - 1 + arglen + 1;
						continue;
					} else if (linebuf[i+1] == '[') {
						size_t arglen = strcspn(linebuf+i,"]");
						i += arglen;
					} else goto _default; 
					break;
				case '\n':
					l++;
					c = _gidis_view_layout[tab];
					assert(l < BUFFER_LINES);
					break;
				case '\t':
					l = baseline;
					c = _gidis_view_layout[++tab];
					break;
				default:;
				_default:;
					assert( l < BUFFER_LINES);
					assert( c < BUFFER_COLUMNS);
					if (c >= 0) 
						buffer[l][c++] = linebuf[i];
					break;
			}
		}

		if (f == NULL) return 0;

		/* Dump chomped lines to the stream */
		int np = 0;
		for (int i = 0; i < BUFFER_LINES; i++) {
			gidis_chomp(buffer[i]);
			if (buffer[i][0] != 0) {
				while (np) fprintf(f, "\n"), np--;
				fprintf(f, "%s\n", buffer[i]);
				wlines++;
			} else np++;
		}
	}
	return wlines;
}

/* 
 * Dumps instructions in range [first,last] into open filedescriptor f.
 * Returns number of line written.
 */
int gidis_dump(FILE *f, int first, int last) {
	assert(f);
	return _gidis_dump(f, first, last);
}

/* 
 * Dumps single instruction and returns pointer to the buffer 
 */
char **gidis_dump_one(int i) {
	_gidis_dump(NULL, i, i);
	return (char **)buffer;
}
