/******************************************************************************
 * Filename: editor.c
 *
 * Created: 2012/03/18 17:20
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include <gtk/gtk.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>

GString *_gidis_editor_text = NULL;
int _gidis_editor_h = 1, _gidis_editor_w = 0;
int _gidis_editor_curpos;

void gidis_editor_size(int height, int width) {

}

void gidis_editor_start_text(char *text) {
	assert(_gidis_editor_text == NULL);
	_gidis_editor_text = g_string_new(text);
	_gidis_editor_curpos = strlen(text);
}

void gidis_editor_start() {
	assert(_gidis_editor_text == NULL);
	_gidis_editor_text = g_string_sized_new( 512 );
	_gidis_editor_curpos = 0;
}

GString *gidis_editor_end() {
	GString *ret = _gidis_editor_text;
	_gidis_editor_text = NULL;
	return ret;
}

gboolean gidis_editor_is_active() {
	return (_gidis_editor_text == NULL) ? FALSE : TRUE;
}

void _gidis_editor_key(int keyval) {
	switch (keyval) {
		case GDK_KEY_Left:
			if (_gidis_editor_curpos > 0) _gidis_editor_curpos--; break;
		case GDK_KEY_Right:
			if (_gidis_editor_curpos < _gidis_editor_text->len) _gidis_editor_curpos++; break;
		case GDK_KEY_Delete:
			if (_gidis_editor_curpos < _gidis_editor_text->len)
				g_string_erase(_gidis_editor_text, _gidis_editor_curpos, 1);
			break;
		case GDK_KEY_End: _gidis_editor_curpos = _gidis_editor_text->len;	break;
		case GDK_KEY_Home: _gidis_editor_curpos = 0;	break;
		case GDK_KEY_BackSpace:
			if (_gidis_editor_curpos > 0)
				g_string_erase(_gidis_editor_text, --_gidis_editor_curpos, 1);
			break;
		default:;
			unsigned int c = gdk_keyval_to_unicode(keyval);
			if (c != 0)
				g_string_insert_unichar(_gidis_editor_text, _gidis_editor_curpos++, c);
	}
}

void _gidis_editor_draw(cairo_t *cr, int x, int y) {
	assert(_gidis_editor_text);

	/* Compute text metrics */
	gchar *pre = malloc(sizeof(gchar *)*(_gidis_editor_curpos+1));
	g_strlcpy(pre, _gidis_editor_text->str, _gidis_editor_curpos+1);

	cairo_text_extents_t ex;
	cairo_text_extents(cr, pre, &ex);
	int curx = x + ex.x_advance;
	cairo_text_extents(cr, "_", &ex);
	int curw = ex.width;

	cairo_font_extents_t fx;
	cairo_font_extents(cr, &fx);

	free(pre);

	/* Print the stuff */
	cairo_move_to(cr, x, y);
	cairo_show_text(cr, _gidis_editor_text->str);

	/* Draw cursor */
	cairo_pattern_t *pt = cairo_get_source(cr);
	cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
	cairo_set_operator(cr, CAIRO_OPERATOR_DIFFERENCE);
	cairo_rectangle(cr, curx, y-fx.ascent, curw, fx.height);
	cairo_fill(cr);
	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
	cairo_set_source(cr, pt);
}
