/******************************************************************************
 * Filename: hotkeys.c *
 * Created: 2012/05/24 20:34
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include "view.h"
#include "editor.h"
#include "command.h"
#include "../config.h"

struct hotkey_item *_gidis_hk_items;

struct saved_state {
	int top;
	int step;
	int cursor;
};

struct saved_state _gidis_states[256];
int _gidis_grab;

gboolean gidis_key_handler(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	int h = gtk_widget_get_allocated_height(widget);
	int lines = h/_gidis_view_line_height;

	gidis_view_hide_message();
	if (gidis_editor_is_active()) {
		switch (event->keyval) {
			case GDK_KEY_Return:;
				GString *cmd = gidis_editor_end();
				gchar *error = gidis_command_execute(_gidis_prg, cmd->str); 
				if (error != NULL)
					gidis_view_show_message_enc(error);
				g_string_free(cmd, TRUE);
				break;
			case GDK_KEY_Escape:	g_string_free(gidis_editor_end(), TRUE); break;
			case GDK_KEY_c:
				if (event->state & GDK_CONTROL_MASK) {
					g_string_free(gidis_editor_end(), TRUE); 
					break;
				}
			default:
				_gidis_editor_key(event->keyval);
		}
		gtk_widget_queue_draw(widget);
		return TRUE;
	} else if (_gidis_grab) {
		int key = event->keyval;
		if (_gidis_grab >= 255) return FALSE;
		if (_gidis_grab == 'm') {
			_gidis_states[key].top = _gidis_view_state.top;
			_gidis_states[key].step = _gidis_view_state.step;
			_gidis_states[key].cursor = _gidis_view_cursor;
		} else if (_gidis_grab == 0x27) {
		
			 _gidis_view_state.top  = _gidis_states[key].top;
			 _gidis_view_state.step = _gidis_states[key].step;
			 _gidis_view_cursor     = _gidis_states[key].cursor;
		}
		gtk_widget_queue_draw(widget);
		_gidis_grab = 0;
		return TRUE;
	} else switch (event->keyval) {
		case GDK_KEY_Page_Up:	_gidis_do_scroll(widget, lines);	break;
		case GDK_KEY_Page_Down:	_gidis_do_scroll(widget, -lines);	break;
		case GDK_KEY_j:			
			if (_gidis_view_cursor < lines-2) _gidis_view_cursor++;
			else _gidis_do_scroll(widget, -1);	
			gtk_widget_queue_draw(widget);
			break;
		case GDK_KEY_k:			
			if (_gidis_view_cursor > 0) _gidis_view_cursor--;
			else _gidis_do_scroll(widget, 1);
			gtk_widget_queue_draw(widget);
			break;
		case GDK_KEY_J:			_gidis_do_scroll(widget, -1);	break;
		case GDK_KEY_K:			_gidis_do_scroll(widget, 1);	break;
		case GDK_KEY_H:			_gidis_view_cursor = 0; 		break;
		case GDK_KEY_M:			_gidis_view_cursor = lines/2; 	break;
		case GDK_KEY_L:			_gidis_view_cursor = lines-2; 	break;
		case GDK_KEY_n:			gidis_view_search_next();		break;
		case GDK_KEY_p:			gidis_view_search_prev();		break;
		case GDK_KEY_v:			gidis_view_toggle_style();		break;
		case GDK_KEY_m:			_gidis_grab = 'm';				break;
		case GDK_KEY_apostrophe:_gidis_grab = 0x27;				break;
		case GDK_KEY_A:		
			CONFIG_TOGGLE(gidis_show_aliases);		
			gidis_view_rebuild_linebuf();
			break;
		case GDK_KEY_C:			gidis_view_clear_style(); 		break;
		case GDK_KEY_B:			gidis_view_toggle_bytes(); 	break;
		case GDK_KEY_exclam:
		case GDK_KEY_slash:
		case GDK_KEY_colon:		
			assert(!gidis_editor_is_active());
			gidis_editor_start(); 
			_gidis_editor_key(event->keyval); 
			gtk_widget_queue_draw(widget);
			break;
		default:;
			/* Try registered handlers. */
			struct hotkey_item *hki = _gidis_hk_items;
			while (hki != NULL) {
				if ((event->keyval == hki->key) && ((event->state & hki->mask) == hki->mask)) {
					PyObject *args = PyTuple_New(2);
					PyTuple_SetItem(args, 0, idis_python_export_program(_gidis_prg));
					PyTuple_SetItem(args, 1, idis_python_export_instr(gidis_view_get_selected()));
					PyObject_CallObject(hki->callback.python, args);
					PyErr_Print();
					break;
				}
				hki = hki->next;
			}
			if (hki == NULL) return FALSE;
	}
	gtk_widget_queue_draw(widget);

	return TRUE;
}

void gidis_view_add_hotkey_hook( char *desc, PyObject *callback ) {
	char *d = desc;
	unsigned int mask = 0;
	char key;
	while (d[0] != 0) {
		if (d[1] == 0)
			key = d[0];
		else switch (d[0]) {
			case '-': break;
			case 'C':; mask |= GDK_CONTROL_MASK; break;
			case 'A':; mask |= GDK_MOD1_MASK; 	break;
			case 'S':; mask |= GDK_SHIFT_MASK;	break;
			default:
				debugf("Invalid character '%c' in key specifier '%s'", d[0], desc);
		}
		d++;
	}
	Py_INCREF(callback);
	struct hotkey_item *item = malloc(sizeof(struct hotkey_item));
	item->next = _gidis_hk_items;
	_gidis_hk_items = item;
	item->mask = mask;
	item->callback.python = callback;
	item->key = key;
}
