/******************************************************************************
 * Filename: region.c
 *
 * Created: 2012/03/26 12:49
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#include "region.h"
#include "util.h"

void gidis_region_start(cairo_t *cr, struct region *region) {
	cairo_get_current_point(cr, &region->x1, &region->y1);
	region->y1 -= fx.ascent+1;
}

void gidis_region_end(cairo_t *cr, struct region *region) {
	cairo_get_current_point(cr, &region->x2, &region->y2);
	region->y2 += fx.descent+1;
}

void gidis_region_invert(cairo_t *cr, struct region *region) {
	gidis_util_invert(cr, region->x1-1, region->y1-1, region->x2+1, region->y2+1);
}

struct region *gidis_region_at(int x, int y) {
	int row = y / 14;
	struct instr_info *info = _gidis_view_state.linemap[row];
	assert(info);
	struct region *r = info->regionlist;
	while (r != NULL) {
		if (r->x1 <= x && r->x2 >= x && r->y1 <= y && r->y2 >= y) break;
		r = r->next;
	}
	return r;
}

struct region *gidis_region_byid(struct instr_info *info, int id) {
	struct region *region = info->regionlist;
	while (region != NULL) {
		if (region->id == id) return region;
		region = region->next;
	}
	return NULL;
}

struct region *gidis_region_new(struct instr_info *info, int id) {
	struct region *region = malloc(sizeof(struct region));
	memset(region, 0, sizeof(struct region));
	region->next = info->regionlist;
	info->regionlist = region;
	region->id = id;
	return region;
}

void gidis_region_free(struct region *region) {
	free(region);
}

void gidis_region_draw(cairo_t *cr, struct instr_info *info, struct region *region ) {
	if (region->style % 2) 
		gidis_region_invert(cr, region);
}
