/******************************************************************************
 * Filename: view.h
 *
 * Created: 2012/03/14 21:41
 * Author: Ladislav Láska
 * e-mail: laska@kam.mff.cuni.cz
 *
 ******************************************************************************/

#ifndef __VIEW_H__
#define __VIEW_H__

#ifndef SWIG

#include "../core.h"
#include <gtk/gtk.h>

GtkWidget *gidis_view_new(struct program *prg);

#endif

void gidis_view_show_message_enc(gchar *message);
void _gidis_view_show_message(char *message, gboolean error);
void gidis_view_hide_message();
void gidis_view_show_message(char *message);
void gidis_view_show_error(char *message);
void gidis_view_rebuild_linebuf();

#ifndef SWIG

struct view_state {
	int top;
	int step;
	struct instr_info **linemap;
	int lines;
	int moused_line;
};

enum {
	GRT_GENERIC,
	GRT_MNEMONIC,
	GRT_OPERAND
};

struct region {
	int id;
	int type;
	double x1,y1,x2,y2;	/* Zone rectangle */
	int style;
	/* TODO: callback if clickable */
	struct region *next;
};

struct instr_info {
	struct instr *instr;
	struct section *sec;
	int lines;
	int baseline;
	int firstline;
	char *linebuf;
	struct region *regionlist;	/* Linked list of zones */
};

enum {
	GST_ADDR = 1,
	GST_FULLTEXT = 2
};

struct search_mark {
	int info;	
	int type;
	short int hi_off;
	short int hi_start;
	short int hi_end;
};

struct hotkey_item {
	char key;
	int mask;
	union {
		PyObject *python;
	} callback;
	struct hotkey_item *next;
};

extern struct instr_info *_gidis_view_instr_info;
extern int _gidis_view_instr_info_len;
extern int _gidis_view_layout[];
extern char *_gidis_view_layout_names[];
extern struct program *_gidis_prg;
extern int _gidis_view_cursor;
extern int _gidis_view_layout_active;
extern int _gidis_view_line_height;

void gidis_view_search_mark(int info, int type);
void gidis_view_search_mark_hi( int info, int type, int hi_off, int hi_start, int hi_end );
void gidis_view_scroll_to( int index );
int _gidis_do_scroll(GtkWidget *widget, int amount);
void gidis_view_search_first();
void gidis_view_search_new();
void gidis_view_search_next();
void gidis_view_search_prev();
void gidis_view_prg_load(struct program *prg);
void gidis_view_prg_free();

void gidis_view_clear_style();
void gidis_view_toggle_style();
void gidis_view_toggle_bytes();

struct instr *gidis_view_get_selected();

extern struct view_state _gidis_view_state;

#endif

#define M_ERR "\x1b"
#define M_MSG ""

#endif
