/******************************************************************************
 * Filename: disasm.c
 * Description:
 *
 * Version: 1.0
 * Created: Sep 11 2011 19:56:47
 * Last modified: Sep 11 2011 19:56:47
 *
 * Author: Ladislav Láska
 * e-mail: ladislav.laska@gmail.com
 *
 ******************************************************************************/

#include "core.h"
#include "gui.h"
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


int main(int argc, char **argv) {
	/* Python is initialized first. It won't work without it. */;
	idis_core_init();

	/* Parse commandline arguments */
	int opt;

	int opt_autoload = 1;

	while ((opt = getopt(argc, argv, "nm:p:h")) != -1) {
		switch (opt) {
			case 'n': opt_autoload = 0; break;
			case 'm': 
				if (idis_python_load_module(optarg) == -1) {
					fprintf(stderr, "Could not load plugin '%s'.\n", optarg);
					return 66;
				}
				break;
			case 'p':;
				idis_python_append_path(optarg, 1);
				break;
			case 'h':
			default:
				fprintf(stderr, "Usage: %s [[-n] [-p path] [-m module] ...]\n", argv[0]);
		}
	}

	idis_python_append_path( PYTHON_PREFIX "/share/idis/python", opt_autoload);
	idis_python_append_path( "/usr/local/share/idis/python", opt_autoload);
	idis_python_append_path( "/usr/share/idis/python", opt_autoload);
	idis_python_append_path( ".", opt_autoload);

	/* Load program configuration. */
	idis_config_load();

	struct program *prg = NULL;
	if (argc > optind) {
		prg = idis_disassemble_file(argv[optind]);
		if (prg != NULL)
			idis_python_call_hooks(prg, 0);
		else 
			fprintf(stderr, "File '%s' not found.", argv[optind]);
	}
	
	
	gidis_main(argc, argv, prg);

	idis_core_fini();
	return 0;
}
