import idis

def make_symbol(prg, instr):
	symbol = prg.add_symbol()
	assumption = prg.assumption_getfor(instr, idis.AK_REG, instr.ops[0].val);
	if assumption == None or assumption.value_type != idis.AV_IMM: return
	opaddr = instr.ops[0].disp + assumption.value.imm;
	dest_sym = prg.lookup_symbol( opaddr )
	if dest_sym != None:
		symbol.addr = instr.vmaddr
		symbol.name = idis.new_str("%s@plt" % dest_sym.name)
		

def instr_hook(prg, section, instr):
	if section.name == ".plt":
		if (instr._def.flags & idis.IDF_BRANCH) and (len(instr.ops) == 1) and (instr.ops[0].type == idis.OT_INDIRECT_DISP):
			make_symbol(prg, instr)

def disassemble_hook(prg):
	plt = prg.lookup_section_by_name(".plt")
	symbol = prg.lookup_symbol( plt.addr )
	if symbol != None and symbol.name == "": 
		symbol.name = "plt_lazyload"

def finalize_hook(prg):
	idis.finalize_symbols(prg)
	


def run(prg):
	plt = None
	for sec in prg.sections:
		if sec.name == ".plt":
			plt = sec
	
	if plt == None:
		print "Could not find plt section to decode."


	for instr in plt.instrs:
		if instr._def.flags & idis.IDF_BRANCH:
			if len(instr.ops) != 1:  continue
			if instr.ops[0].type != idis.OT_INDIRECT_DISP: continue
			make_symbol(prg, instr)
	
	idis.finalize_symbols(prg)
