import idis
import gidis

def init_hook():
	gidis.add_context_hook("What is it?", gidis.CMT_OPERAND, whatis_callback);
	gidis.add_context_hook("What is it?", gidis.CMT_MNEMONIC, whatis_mnem);

def whatis_mnem(program, instr):
	gidis.show_message("""Instruction, mnemonics: %(mnem)s (%(desc)s)""" % dict(mnem=instr.mnem,desc=instr.desc,modrm=instr.modrm, sib=instr.sib));

def mem_contents(program, vmaddr):
	# Try read a string
	s = idis.mem_read_string(program, vmaddr);
	if s != None: 
		return ''.join(["'",s.replace("\n", '\\n'),"'"]);
	else: 
		return "%s" % idis.strimm(idis.mem_read_long(program, vmaddr));
	

def whatis_callback(program, instr, op):
	ex = op.expr;
	simple = idis.expr_simplify(ex, program, instr);
	val = idis.expr_enumerate(ex, program, instr);
	signed = idis.signed(val);

	if not idis.expr_enumerate_error():
		sec = idis.lookup_section_by_addr(program, val);
		sym = idis.lookup_symbol_nearest(program, val);
		msg = [];
		if sec != None:
			msg.append("Value %(val)s, valid address in section '%(sec)s': %(sym)s." % dict(val=simple,sec=sec.name,sym=idis.strsym_rel(sym,val)));
			msg.append("\nInitial value: %s" % mem_contents(program, val));
		else:
			msg.append("Value %(hex)s (dec=%(dec)i), not in initial address apce." % dict(hex=idis.strimm(signed),dec=signed));
		gidis.show_message(''.join(msg));
	elif ex.type == idis.EX_REG:
		# If value was assumed, the simplification algorithm would resolve it.
		gidis.show_message("Register %s, no assumed values." % str(ex));
	elif ex.type == idis.EX_DEREF:
		# Handle simple dereferences.
		val = idis.expr_enumerate(simple.ops[0], program, instr);
		sec = idis.lookup_section_by_addr(program, val);
		sym = idis.lookup_symbol_nearest(program, val);
		msg = [ "Reference [%s]" % simple.ops[0]]
		if sec != None:
			msg.append(	", valid address in section '%s'" % sec.name);
			if sym != None:
				msg.append( ", near symbol %s" % idis.strsym_rel(sym,val) );
			msg.append("\nInitial value: %s" % mem_contents(program, val));
		else:
			msg.append( ", not in initial address space" );
		msg.append(".");
		gidis.show_message(''.join(msg));

	else:
		gidis.show_message("Expression: %s" % str(ex));
	
	simple.free();
